/*
 * plugins
 * Copyright (C) 2006 Johan Enell
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package basefile;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

public class BASEFileSpotSection<R, S> extends BASEFileSection
{

	private int height;
	private int width;
	private ArrayList<S> spotData;
	private ArrayList<R> reporterData;
	
	public BASEFileSpotSection()
	{
		super();
	}

	public void setDataMatrix(int height, int width)
	{
		this.height = height;
		this.width = width;
		this.spotData = new ArrayList<S>(height*width);
		this.reporterData = new ArrayList<R>(height);
	}
	
	public void sortSpot(Comparator<S> comparator)
	{
		Collections.sort(spotData, comparator);
	}

	public void sortReporter(Comparator<R> comparator)
	{
		Collections.sort(reporterData, comparator);
	}
	
	public Iterator<S> spotIterator()
	{
		return spotData.iterator();
	}

	public boolean addData(R reporter, S... spots)
	{
		if (spotData == null || reporterData == null) return false;
		if (spots.length != width) return false;
		return reporterData.add(reporter) && spotData.addAll(Arrays.asList(spots));
	}
	
	public void set(int i, R reporter, S... spots)
	{
		if (spotData == null || reporterData == null) return;
		if (spots.length != width) return;
		reporterData.set(i, reporter);
		i = i * width;
		for (S s : spots)
		{
			spotData.set(i++, s);
		}
	}
	
	public S setSpot(R reporter, int i, S spot)
	{
		if (spotData == null) return null;
		if (i >= width) return null;
		i = reporterData.indexOf(reporter) + i;
		return spotData.set(i, spot);
		
	}
	
	public R setReporter(int i, R reporter)
	{
		if (reporterData == null) return null;
		return reporterData.set(i, reporter);
	}
	
	public List<String> getColumnHeader()
	{
		return this.findFieldList("columns");
	}

	public List<Integer> getAssaysHeader()
	{
		return this.findFieldIntList("assays");
	}

	public List<String> getAssayFieldsHeader()
	{
		return this.findFieldList("assayFields");
	}
	
	public int getCountHeader()
	{
		return this.findIntOpt("count");
	}
	
	public boolean isValid()
	{
		if (!isType("spots")) return false;
		if (getColumnHeader() == null) return false;
		if (getAssayFieldsHeader() == null) return false;
		if (getAssaysHeader() == null) return false;
		return true;
	}

	public final int getHeight()
	{
		return height;
	}

	public final int getWidth()
	{
		return width;
	}

	public R getReporter(int i)
	{
		return reporterData.get(i);
	}

	public S getSpot(int i)
	{
		return spotData.get(i);
	}

	public List<S> spotSubList(int fromIndex, int toIndex)
	{
		return spotData.subList(fromIndex, toIndex);
	}

	public List<R> reporterSubList(int fromIndex, int toIndex)
	{
		return reporterData.subList(fromIndex, toIndex);
	}
	
	public int getSpotSize()
	{
		return spotData.size();
	}

	public int getReporterSize()
	{
		return reporterData.size();
	}
}
